﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Modbus.Device;
using System.Windows.Forms;
using System.Timers;
using ModbusDemo.windows;
using ModbusDemo;

namespace ModbusDemo.MessageFormat
{
    public static class ModbusComm
    {
        public  enum eModbusErrorCode : byte
        {
            MB_ENOERR,                  /*!< no error. */
            MB_ENOREG,                  /*!< illegal register address. */
            MB_EINVAL,                  /*!< illegal argument. */
            MB_EPORTERR,                /*!< porting layer error. */
            MB_ENORES,                  /*!< insufficient resources. */
            MB_EIO,                     /*!< I/O error. */
            MB_EILLSTATE,               /*!< protocol stack in illegal state. */
            MB_ETIMEDOUT,               /*!< timeout error occurred. */
            MB_EMASTERNOERR,            /*!< master mode no error. */
            MB_EMASTERERR,              /*!< master mode error. */
            MB_EMASTERSENT,
            MB_RTUERNOERR
        }

        public  static ModbusMaster Modbus;
        public  static ushort usCommunFailTimes = 0;
        private static object oLock = new object();
        public static eModbusErrorCode Modbus_ReadHoldingRegistersTask(out ushort[] OutputValue, byte slaveAddress, ushort startAddress, ushort numberOfPoints)
        {
            lock (oLock)
            {
                eModbusErrorCode ModbusErrorCode;
                try
                {
                    OutputValue = Modbus.ReadHoldingRegisters(slaveAddress, startAddress, numberOfPoints);
                    ModbusErrorCode = eModbusErrorCode.MB_EMASTERNOERR;
                }
                catch (Exception exception)
                {
                    //Connection exception
                    //No response from server.
                    //The server maybe close the com port, or response timeout.
                    if (exception.Source.Equals("System"))
                    {
                        Console.WriteLine(DateTime.Now.ToString() + " " + exception.Message);
                        Form1.totxt.Log(DateTime.Now.ToString() + " " + exception.Message + "Modbus_ReadHoldingRegistersTask");
                        OutputValue = new ushort[numberOfPoints];
                        ModbusErrorCode = eModbusErrorCode.MB_ETIMEDOUT;
                        usCommunFailTimes++;
                        Console.WriteLine(slaveAddress.ToString());
                    }
                    //The server return error code.
                    //You can get the function code and exception code.
                    if (exception.Source.Equals("nModbusPC"))
                    {
                        string str = exception.Message;
                        int FunctionCode;
                        string ExceptionCode;

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        FunctionCode = Convert.ToInt16(str.Remove(str.IndexOf("\r\n")));
                        Console.WriteLine("Function Code: " + FunctionCode.ToString("X"));
                        Form1.totxt.Log("Function Code: " + FunctionCode.ToString("X"));

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        ExceptionCode = str.Remove(str.IndexOf("-"));
                        switch (ExceptionCode.Trim())
                        {
                            case "1":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                            break;
                            case "2":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                            break;
                            case "3":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                            break;
                            case "4":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                            break;
                            case "5":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                            break;
                            case "6":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                            break;
                            case "8":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                            break;
                            case "A":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                            break;
                            case "B":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                            break;
                        }
                    }
                    OutputValue = new ushort[numberOfPoints];
                    ModbusErrorCode = eModbusErrorCode.MB_EMASTERERR;
                }
                return ModbusErrorCode;
            }
        }
        public static eModbusErrorCode Modbus_ReadCoilsTask(out bool[] OutputValue, byte slaveAddress, ushort startAddress, ushort numberOfPoints)
        {
            lock (oLock)
            {
                eModbusErrorCode ModbusErrorCode;
                try
                {
                    OutputValue = Modbus.ReadCoils(slaveAddress, startAddress, numberOfPoints);
                    ModbusErrorCode = eModbusErrorCode.MB_EMASTERNOERR;
                }
                catch (Exception exception)
                {
                    //Connection exception
                    //No response from server.
                    //The server maybe close the com port, or response timeout.
                    if (exception.Source.Equals("System"))
                    {
                        Console.WriteLine(DateTime.Now.ToString() + " " + exception.Message);
                        Form1.totxt.Log(DateTime.Now.ToString() + " " + exception.Message + "Modbus_ReadCoilsTask");
                        OutputValue = new bool[numberOfPoints];
                        ModbusErrorCode = eModbusErrorCode.MB_ETIMEDOUT;
                        usCommunFailTimes++;
                        Console.WriteLine(slaveAddress.ToString());
                    }
                    //The server return error code.
                    //You can get the function code and exception code.
                    if (exception.Source.Equals("nModbusPC"))
                    {
                        string str = exception.Message;
                        int FunctionCode;
                        string ExceptionCode;

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        FunctionCode = Convert.ToInt16(str.Remove(str.IndexOf("\r\n")));
                        Console.WriteLine("Function Code: " + FunctionCode.ToString("X"));
                        Form1.totxt.Log("Function Code: " + FunctionCode.ToString("X"));

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        ExceptionCode = str.Remove(str.IndexOf("-"));
                        switch (ExceptionCode.Trim())
                        {
                            case "1":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                            break;
                            case "2":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                            break;
                            case "3":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                            break;
                            case "4":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                            break;
                            case "5":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                            break;
                            case "6":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                            break;
                            case "8":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                            break;
                            case "A":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                            break;
                            case "B":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                            break;
                        }
                    }
                    OutputValue = new bool[numberOfPoints];
                    ModbusErrorCode = eModbusErrorCode.MB_EMASTERERR;
                }
                return ModbusErrorCode;
            }
        }
        public static eModbusErrorCode Modbus_WriteSingleCoil(byte slaveAddress, ushort startAddress, bool value)
        {
            lock (oLock)
            {
                eModbusErrorCode ModbusErrorCode;
                try
                {
                    Modbus.WriteSingleCoil(slaveAddress, startAddress, value);
                    ModbusErrorCode = eModbusErrorCode.MB_EMASTERNOERR;
                }
                catch (Exception exception)
                {
                    //Connection exception
                    //No response from server.
                    //The server maybe close the com port, or response timeout.
                    if (exception.Source.Equals("System"))
                    {
                        Console.WriteLine(DateTime.Now.ToString() + " " + exception.Message);
                        Form1.totxt.Log(DateTime.Now.ToString() + " " + exception.Message + "Modbus_WriteSingleCoil");
                        ModbusErrorCode = eModbusErrorCode.MB_ETIMEDOUT;
                        usCommunFailTimes++;
                        Console.WriteLine(slaveAddress.ToString());
                    }
                    //The server return error code.
                    //You can get the function code and exception code.
                    if (exception.Source.Equals("nModbusPC"))
                    {
                        string str = exception.Message;
                        int FunctionCode;
                        string ExceptionCode;

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        FunctionCode = Convert.ToInt16(str.Remove(str.IndexOf("\r\n")));
                        Console.WriteLine("Function Code: " + FunctionCode.ToString("X"));
                        Form1.totxt.Log("Function Code: " + FunctionCode.ToString("X"));

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        ExceptionCode = str.Remove(str.IndexOf("-"));
                        switch (ExceptionCode.Trim())
                        {
                            case "1":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                            break;
                            case "2":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                            break;
                            case "3":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                            break;
                            case "4":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                            break;
                            case "5":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                            break;
                            case "6":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                            break;
                            case "8":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                            break;
                            case "A":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                            break;
                            case "B":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                            break;
                        }
                    }
                    ModbusErrorCode = eModbusErrorCode.MB_EMASTERERR;
                }
                return ModbusErrorCode;
            }
        }
        public static eModbusErrorCode Modbus_WriteMultipleCoils(byte slaveAddress, ushort startAddress, bool[] data)
        {
            lock (oLock)
            {
                eModbusErrorCode ModbusErrorCode;
                try
                {
                    Modbus.WriteMultipleCoils(slaveAddress, startAddress, data);
                    ModbusErrorCode = eModbusErrorCode.MB_EMASTERNOERR;
                }
                catch (Exception exception)
                {
                    //Connection exception
                    //No response from server.
                    //The server maybe close the com port, or response timeout.
                    if (exception.Source.Equals("System"))
                    {
                        Console.WriteLine(DateTime.Now.ToString() + " " + exception.Message);
                        Form1.totxt.Log(DateTime.Now.ToString() + " " + exception.Message + "Modbus_WriteMultipleCoils");
                        ModbusErrorCode = eModbusErrorCode.MB_ETIMEDOUT;
                        usCommunFailTimes++;
                        Console.WriteLine(slaveAddress.ToString());
                    }
                    //The server return error code.
                    //You can get the function code and exception code.
                    if (exception.Source.Equals("nModbusPC"))
                    {
                        string str = exception.Message;
                        int FunctionCode;
                        string ExceptionCode;

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        FunctionCode = Convert.ToInt16(str.Remove(str.IndexOf("\r\n")));
                        Console.WriteLine("Function Code: " + FunctionCode.ToString("X"));
                        Form1.totxt.Log("Function Code: " + FunctionCode.ToString("X"));

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        ExceptionCode = str.Remove(str.IndexOf("-"));
                        switch (ExceptionCode.Trim())
                        {
                            case "1":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                            break;
                            case "2":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                            break;
                            case "3":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                            break;
                            case "4":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                            break;
                            case "5":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                            break;
                            case "6":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                            break;
                            case "8":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                            break;
                            case "A":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                            break;
                            case "B":
                            Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                            Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                            break;
                        }
                    }
                    ModbusErrorCode = eModbusErrorCode.MB_EMASTERERR;
                }
                return ModbusErrorCode;
            }
        }
    }
}
